@@ -1,5 +1,6 @@
 """Detector geometry handling."""
 import warnings
+from copy import copy
 from itertools import product
 from typing import List, Tuple
 
@@ -7,7 +8,7 @@ import h5py
 import numpy as np
 
 from .base import DetectorGeometryBase, GeometryFragment
-
+from .snapped import isinstance_no_import
 
 class GenericGeometry(DetectorGeometryBase):
     """A generic detector layout based either on the CrystFEL geom file or on a set of parameters.
@@ -1389,6 +1390,98 @@ class DSSC_1MGeometry(DetectorGeometryBase):
             ax.set_aspect(204/236.)
         return ax
 
+    def plot_data_hexes(
+            self, data, *, frontview=True, ax=None, figsize=None, colorbar=False,
+            vmin=None, vmax=None, norm=None, cmap=None, module=None,
+    ):
+        """Plot data from the detector showing hexagonal pixels
+
+        Most of the arguments are like those for :meth:`plot_data`. This method
+        is slower, but sometimes useful to look at small details. It can also
+        plot data for a single module, if you pass a suitable 2D array as *data*
+        and a module number as *module*.
+        """
+        import matplotlib.pyplot as plt
+        from matplotlib.cm import viridis
+        from matplotlib.collections import PolyCollection
+
+        single_mod = module is not None
+        if single_mod:
+            assert data.shape == self.expected_data_shape[1:]
+        else:
+            assert data.shape == self.expected_data_shape
+            module = np.s_[:]
+
+        if norm is not None and not (vmax is None and vmin is None):
+            raise ValueError("Pass norm or vmin/vmax, not both")
+
+        px_offsets = self.get_pixel_positions(centre=False)[module, ..., :2]\
+                         .reshape(-1, 2)
+
+        min_x, min_y = px_offsets.min(axis=0)
+        max_x, max_y = px_offsets.max(axis=0)
+
+        cross_size = 20 * self.pixel_size
+
+        # Use a dark grey for missing data
+        if cmap is None:
+            cmap = copy(viridis)
+            cmap.set_bad('0.25', 1.0)
+
+        if ax is None:
+            fig = plt.figure(figsize=figsize or (
+                (8, 3) if single_mod else (10, 10)
+            ))
+            ax = fig.add_subplot(1, 1, 1)
+
+        try:
+            # matplotlib 3.3.0 onwards
+            from matplotlib.transforms import AffineDeltaTransform
+            tfm_kwargs = {'transOffset': AffineDeltaTransform(ax.transData)}
+        except ImportError:
+            # Works Up to (& excluding) matplotlib 3.5.0 - offset_position removed
+            from matplotlib.transforms import IdentityTransform
+            tfm_kwargs = {
+                'transOffset': IdentityTransform(), 'offset_position': 'data'
+            }
+
+        collection = PolyCollection(
+            [self._pixel_corners[::-1].T * self.pixel_size],
+            offsets=px_offsets,
+            cmap=cmap,
+            norm=norm,
+            **tfm_kwargs
+        )
+        collection.set_array(data.ravel())
+        if vmin is not None or vmax is not None:
+            collection.set_clim(vmin, vmax)
+
+        ax.add_collection(collection)
+        ax.set_facecolor('0.25')
+
+        if isinstance(colorbar, dict) or colorbar is True:
+            if isinstance(colorbar, bool):
+                colorbar = {}
+            if single_mod and ('location' not in colorbar):
+                # With single module, horizontal colorbar uses space better
+                colorbar.setdefault('orientation', 'horizontal')
+            plt.colorbar(collection, ax=ax, **colorbar)
+
+        ax.set_xlabel('metres')
+        ax.set_ylabel('metres')
+
+        margin = 3 * self.pixel_size
+        ax.set_xlim(min_x - margin, max_x + margin)
+        ax.set_ylim(min_y - margin, max_y - margin)
+        ax.set_aspect(1)
+        if frontview:
+            ax.invert_xaxis()
+
+        # Draw a cross at the centre
+        ax.hlines(0, -cross_size, +cross_size, colors='w', linewidths=1)
+        ax.vlines(0, -cross_size, +cross_size, colors='w', linewidths=1)
+        return ax
+
     @classmethod
     def _tile_slice(cls, tileno):
         tile_offset = tileno * cls.frag_fs_pixels
@@ -1423,7 +1516,7 @@ class DSSC_1MGeometry(DetectorGeometryBase):
     @classmethod
     def _adjust_pixel_coords(cls, ss_coords, fs_coords, centre):
         # Shift odd-numbered rows by half a pixel.
-        fs_coords[1::2] -= 0.5
+        fs_coords[1::2] += 0.5
         if centre:
             # Vertical (slow scan) centre is 2/3 of the way to the start of the
             # next row of hexagons, because the tessellating pixels extend
@@ -1534,7 +1627,7 @@ class JUNGFRAUGeometry(DetectorGeometryBase):
             modules.append(tiles)
         return cls(modules)
 
-    def inspect(self, axis_units='px', frontview=True):
+    def inspect(self, axis_units='px', frontview=True, module_names=[]):
         """Plot the 2D layout of this detector geometry.
 
         Returns a matplotlib Axes object.
@@ -1551,14 +1644,19 @@ class JUNGFRAUGeometry(DetectorGeometryBase):
         ax = super().inspect(axis_units=axis_units, frontview=frontview)
         scale = self._get_plot_scale_factor(axis_units)
 
-        for m in range(len(self.modules)):
-            tiles = self.modules[m]
+        for i, tiles in enumerate(self.modules):
+            cx, cy, _ = tiles[2].centre() * scale
+            module_text = module_names[i] if module_names else f'JNGFR{i+1:02d}'
+            ax.text(cx, cy, module_text, fontweight='bold',
+                    size='x-large',
+                    verticalalignment='center',
+                    horizontalalignment='center')
 
-            # Label tiles in the module: A0 to A8
-            for t, tile in enumerate(tiles):
-                s = 'M{M}A{T}'.format(T=t, M=m)
-                cx, cy, _ = tile.centre() * scale
-                ax.text(cx, cy, s, fontweight='bold',
+            # Label tiles in the module for 1st and last ASICs only.
+            for t in [0, 4, 7]:
+                s = 'A{T}'.format(T=t)
+                cx, cy, _ = tiles[t].centre() * scale
+                ax.text(cx, cy, s,
                         verticalalignment='center',
                         horizontalalignment='center')
 
@@ -1585,6 +1683,14 @@ class JUNGFRAUGeometry(DetectorGeometryBase):
         fs_slice = slice(tile_fs_offset, tile_fs_offset + cls.frag_fs_pixels)
         return ss_slice, fs_slice
 
+    def position_modules(self, data, out=None, threadpool=None):
+        if isinstance_no_import(data, 'xarray', 'DataArray'):
+            # we shift module indices by one as JUNGFRAU labels modules starting from 1..
+            # position_modules returns a numpy array so labels disapear anyway.
+            data = data.copy(deep=False)
+            data['module'] = data['module'] - 1
+        return super().position_modules(data, out=out, threadpool=threadpool)
+
 
 class PNCCDGeometry(DetectorGeometryBase):
     """Detector layout for pnCCD
