@@ -31,6 +31,18 @@ def test_inspect():
     assert isinstance(ax, Axes)
 
 
+def test_plot_data_hexes():
+    geom = DSSC_1MGeometry.from_h5_file_and_quad_positions(
+        sample_xfel_geom, QUAD_POS
+    )
+    # Smoketest
+    ax = geom.plot_data_hexes(np.zeros((16, 128, 512)), colorbar=True)
+    assert isinstance(ax, Axes)
+
+    ax = geom.plot_data_hexes(np.zeros((128, 512)), module=4, colorbar=True)
+    assert isinstance(ax, Axes)
+
+
 def test_snap_assemble_data():
     geom = DSSC_1MGeometry.from_h5_file_and_quad_positions(
         sample_xfel_geom, QUAD_POS
@@ -74,8 +86,8 @@ def test_get_pixel_positions():
     assert -0.2 < py.min() < -0.12
     assert  0.2 > py.max() >  0.12
 
-    # Odd-numbered rows in Q1 & Q2 should have 0.5 pixel higher x than the even.
-    np.testing.assert_allclose(px[0, 1::2, 0] - px[0, 0::2, 0], 236e-6/2)
+    # Odd-numbered rows in Q1 & Q2 should have 0.5 pixel lower x than the even.
+    np.testing.assert_allclose(px[0, 1::2, 0] - px[0, 0::2, 0], -236e-6/2)
 
 
 def test_read_write_xfel_file_quadpos(tmpdir):
