@@ -1,13 +1,17 @@
-from cfelpyutils.crystfel_utils import load_crystfel_geometry
+from os.path import abspath, dirname
+from os.path import join as pjoin
+
 import h5py
-from matplotlib.axes import Axes
 import numpy as np
-from os.path import abspath, dirname, join as pjoin
+import pyFAI.detectors
 import pytest
+from cfelpyutils.geometry import load_crystfel_geometry
+from matplotlib.axes import Axes
 from testpath import assert_isfile
 
 from extra_geom import LPD_1MGeometry
 from extra_geom.detectors import invert_xfel_lpd_geom
+
 from .utils import assert_geom_close
 
 tests_dir = dirname(abspath(__file__))
@@ -35,7 +39,7 @@ def test_write_read_crystfel_file(tmpdir):
     np.testing.assert_allclose(loaded.modules[0][0].fs_vec, geom.modules[0][0].fs_vec)
 
 
-    geom_dict = load_crystfel_geometry(path)
+    geom_dict = load_crystfel_geometry(path).detector
     quad_gr0 = ['p0a0', 'p0a1', 'p0a2', 'p0a3', 'p0a4', 'p0a5', 'p0a6', 'p0a7',
                 'p0a8', 'p0a9', 'p0a10','p0a11', 'p0a12', 'p0a13', 'p0a14',
                 'p0a15', 'p1a0', 'p1a1','p1a2', 'p1a3','p1a4','p1a5','p1a6',
@@ -50,7 +54,7 @@ def test_write_read_crystfel_file(tmpdir):
     assert geom_dict['rigid_groups']['q0'] == quad_gr0
 
 
-def test_read_write_xfel_file(tmpdir):
+def test_read_write_xfel_file_quadpos(tmpdir):
     quad_pos = [(11.4, 299), (-11.5, 8), (254.5, -16), (278.5, 275)]
     geom = LPD_1MGeometry.from_quad_positions(quad_pos)
     path = str(tmpdir / 'lpd_geom.h5')
@@ -63,6 +67,18 @@ def test_read_write_xfel_file(tmpdir):
     assert_geom_close(loaded, geom)
 
 
+def test_read_write_xfel_file(tmpdir):
+    quad_pos = [(11.4, 299), (-11.5, 8), (254.5, -16), (278.5, 275)]
+    geom = LPD_1MGeometry.from_quad_positions(quad_pos)
+    path = str(tmpdir / 'lpd_geom.h5')
+    geom.to_h5_file_and_quad_positions(path)
+
+    assert_isfile(path)
+
+    loaded = LPD_1MGeometry.from_h5_file(path)
+    assert_geom_close(loaded, geom)
+
+
 def test_quad_positions_with_file():
     path = pjoin(tests_dir, 'lpd_mar_18.h5')
     quad_pos = [(11.4, 299), (-11.5, 8), (254.5, -16), (278.5, 275)]
@@ -134,6 +150,100 @@ def test_offset():
         geom.offset(np.zeros(4))
 
 
+def test_rotate():
+    quad_pos = [(11.4, 299), (-11.5, 8), (254.5, -16), (278.5, 275)]
+    geom = LPD_1MGeometry.from_quad_positions(quad_pos)
+
+    # Uniform rotation for all modules, all tiles
+    all_rotated = geom.rotate((90, 0, 0))
+    y = np.array([m[0].corner_pos[1] for m in all_rotated.modules])
+    np.testing.assert_allclose(y, 0., atol=1e-7)
+
+    all_rotated = geom.rotate((0, 90, 0))
+    x = np.array([m[0].corner_pos[0] for m in all_rotated.modules])
+    np.testing.assert_allclose(x, 0., atol=1e-7)
+
+    all_rotated = geom.rotate((0, 0, 90))
+    assert all_rotated.output_array_for_position_fast().T.shape == \
+        geom.output_array_for_position_fast().shape
+
+    all_rotated = geom.rotate((90, 0, 0), center=(0, 1, 0))
+    y = np.array([m[0].corner_pos[1] for m in all_rotated.modules])
+    np.testing.assert_allclose(y, 1)
+
+    # Select some modules
+    q4_rotated = geom.rotate((0, 0, 180), modules=np.s_[12:])
+    assert q4_rotated.output_array_for_position_fast().shape == \
+        geom.output_array_for_position_fast().shape
+    np.testing.assert_array_almost_equal(
+        q4_rotated.modules[-1][7].corners(),
+        np.roll(geom.modules[-1][15].corners(), 2, 0)
+    )
+
+    # Per-module rotation
+    rotations = [(0, 0, 90), (0, 0, 180), (90, 0, 0), (0, 90, 0)]
+    q1_rotated = geom.rotate(rotations, modules=np.s_[:4])
+    mod3_center = np.mean([t.centre() for t in geom.modules[2]], axis=0)
+    mod4_center = np.mean([t.centre() for t in geom.modules[3]], axis=0)
+    np.testing.assert_allclose([t.corner_pos[1] for t in q1_rotated.modules[2]], mod3_center[1])
+    np.testing.assert_allclose([t.corner_pos[0] for t in q1_rotated.modules[3]], mod4_center[0])
+
+    # Per-tile rotation
+    rot = np.zeros((4, 16, 3), dtype=np.float64)
+    rot[:, 5, :] = (0, 0, 180)  # rotate T6 of each module
+    q2_t2_rotated = geom.rotate(rot, modules=np.s_[4:8])
+    for mod_rot, mod_ref in zip(q2_t2_rotated.modules[4:8], geom.modules[4:8]):
+        np.testing.assert_array_almost_equal(
+            mod_rot[5].corners(),
+            np.roll(mod_ref[5].corners(), 2, 0)
+        )
+    for mod_rot, mod_ref in zip(q2_t2_rotated.modules[:4], geom.modules[:4]):
+        np.testing.assert_array_almost_equal(
+            mod_rot[5].corners(),
+            mod_ref[5].corners(),
+        )
+
+    # set rotation center, per-det/mod/tile
+    rot = np.zeros((4, 16, 3), dtype=np.float64)
+    rot[:, 2, :] = (90, 0, 0)
+    center = np.zeros((4, 16, 3), dtype=np.float64)
+    center[0, 2, :] = geom.modules[8][2].centre()
+    center[1, 2, :] = geom.modules[9][2].centre()
+    center[2, 2, :] = geom.modules[10][2].centre()
+    center[3, 2, :] = geom.modules[11][2].centre()
+    q3_t2_rotated = geom.rotate(rot, center=center, modules=np.s_[8:12])
+    y = np.array([m[2].corner_pos[1] for m in q3_t2_rotated.modules[8:12]])
+    np.testing.assert_array_almost_equal(
+        y,
+        [
+            geom.modules[8][2].centre()[1],
+            geom.modules[9][2].centre()[1],
+            geom.modules[10][2].centre()[1],
+            geom.modules[11][2].centre()[1]
+        ]
+    )
+
+    # Wrong number of modules
+    with pytest.raises(ValueError):
+        geom.rotate(np.ones((15, 3)))
+
+    # Rotation for 16 modules, but only 4 selected
+    with pytest.raises(ValueError):
+        geom.rotate(np.ones((16, 16, 3)), modules=np.s_[:4])
+
+    # angles must be 3D
+    with pytest.raises(ValueError):
+        geom.rotate((1, 1))
+
+    # angles in radian
+    deg = geom.rotate((90, 0, 0), modules=np.s_[:1], tiles=np.s_[:1])
+    rad = geom.rotate((np.pi / 2, 0, 0), modules=np.s_[:1], tiles=np.s_[:1], degrees=False)
+    np.testing.assert_array_almost_equal(
+        deg.modules[1][1].corners(),
+        rad.modules[1][1].corners()
+    )
+
+
 def test_inspect():
     geom = LPD_1MGeometry.from_quad_positions(
         [(11.4, 299), (-11.5, 8), (254.5, -16), (278.5, 275)]
@@ -209,3 +319,11 @@ def test_invert_xfel_lpd_geom(tmpdir):
         np.testing.assert_array_equal(
             fsrc['Q1/M1/T07/Position'][:], -1 * fdst['Q1/M1/T07/Position'][:]
         )
+
+
+def test_to_pyfai_detector():
+    geom = LPD_1MGeometry.from_quad_positions(
+        [(11.4, 299), (-11.5, 8), (254.5, -16), (278.5, 275)]
+    )
+    agipd_pyfai = geom.to_pyfai_detector()
+    assert isinstance(agipd_pyfai, pyFAI.detectors.Detector)
